/*
 * $Id$
 *
 * SARL is an general-purpose agent programming language.
 * More details on http://www.sarl.io
 *
 * Copyright (C) 2014-2016 the original authors or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.sarl.lang.actionprototype;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.common.types.JvmTypeReference;

/**
 * This class permits to wrap the formal parameters.
 *
 * @author $Author: sgalland$
 * @version $FullVersion$
 * @mavengroupid $GroupId$
 * @mavenartifactid $ArtifactId$
 */
public class InferredStandardParameter {

	/** Original parameter.
	 */
	protected final EObject source;

	/** Name.
	 */
	protected final String name;

	/** Name.
	 */
	protected final JvmTypeReference type;

	/** Default value annotation value.
	 */
	protected String defaultValueAnnotationValue;

	/**
	 * @param source - the original parameter.
	 * @param name - the name of the formal parameter.
	 * @param type - the type of the formal parameter.
	 */
	public InferredStandardParameter(EObject source, String name, JvmTypeReference type) {
		this.source = source;
		this.name = name;
		this.type = type;
	}

	/** Replies the source parameter.
	 *
	 * @return the source parameter.
	 */
	public EObject getParameter() {
		return this.source;
	}

	/** Replies the name.
	 *
	 * @return the name.
	 */
	public String getName() {
		return this.name;
	}

	/** Replies the type of the parameter.
	 *
	 * @return the type.
	 */
	public JvmTypeReference getType() {
		return this.type;
	}

	@Override
	public String toString() {
		return this.name + " : " + this.type; //$NON-NLS-1$
	}

	/** Replies the value of the annotation that is marked this parameter with a default value.
	 *
	 * @return the annotation's value.
	 */
	public String getDefaultValueAnnotationValue() {
		return this.defaultValueAnnotationValue;
	}

	/** Set the value of the annotation that is marked this parameter with a default value.
	 *
	 * @param value the annotation's value.
	 */
	void setDefaultValueAnnotationValue(String value) {
		this.defaultValueAnnotationValue = value;
	}

}
